/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.output
 * File:    JPrefuse.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.output;

/**
 * The Implementation of a Prefuse-Graph
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.awt.event.MouseEvent;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.DataColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.action.layout.graph.ForceDirectedLayout;
import prefuse.activity.Activity;
import prefuse.controls.ControlAdapter;
import prefuse.controls.FocusControl;
import prefuse.controls.NeighborHighlightControl;
import prefuse.controls.PanControl;
import prefuse.controls.SubtreeDragControl;
import prefuse.controls.ToolTipControl;
import prefuse.controls.WheelZoomControl;
import prefuse.controls.ZoomControl;
import prefuse.controls.ZoomToFitControl;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.EdgeRenderer;
import prefuse.render.LabelRenderer;
import prefuse.util.ColorLib;
import prefuse.util.force.DragForce;
import prefuse.util.force.ForceSimulator;
import prefuse.util.force.NBodyForce;
import prefuse.util.force.RungeKuttaIntegrator;
import prefuse.util.force.SpringForce;
import prefuse.visual.VisualItem;
import de.fhhannover.inform.ifmap.control.IfmapFacade;
import de.fhhannover.inform.ifmap.datastructure.Identifier;
import de.fhhannover.inform.ifmap.datastructure.IdentifierContainer;
import de.fhhannover.inform.ifmap.datastructure.Link;
import de.fhhannover.inform.ifmap.datastructure.LinkContainer;
import de.fhhannover.inform.ifmap.event.IdentifierChangedReceiver;
import de.fhhannover.inform.ifmap.event.NodeSelectedReceiver;
import de.fhhannover.inform.ifmap.event.StatusChangedReceiver;

@SuppressWarnings("serial")
public class JPrefuse
extends Display implements ComponentListener, IdentifierChangedReceiver{
	
	private Hashtable<String, Node> ident2node = new Hashtable<String, Node>();
	private Hashtable<String, Identifier> ident2identifier = new Hashtable<String, Identifier>();
	private Hashtable<String, Node> link2node = new Hashtable<String, Node>();
	private Hashtable<String, Link> link2link = new Hashtable<String, Link>();
	private ArrayList<NodeSelectedReceiver> receiver_nodes = new ArrayList<NodeSelectedReceiver>();
	//private Hashtable<Link, Node> linko2node = new Hashtable<Link, Node>();
	//private ArrayList<Link> links = new ArrayList<Link>();
	private Graph graph;
	private Visualization vis;
	private boolean animation = true;
	private VisualItem selectedNode;
	private JTable table;
	private JLabel label;
	
	public JPrefuse(IdentifierContainer icontainer){
		init(icontainer);
	}
	
	public JPrefuse(){
		init(null);
	}
	
	public void setTable(JTable tab, JLabel label){
		this.table = tab;
		this.label = label;
	}
	
	public void addNodeSelectedListener(NodeSelectedReceiver r) {
		if (!receiver_nodes.contains(r))
			receiver_nodes.add(r);
	}
	
	
	public void notifyNodeSelectedReceiver(VisualItem item) {
		for (NodeSelectedReceiver r : receiver_nodes) {
			r.nodeSelected(item);
		}
	}

	private Graph fillGraph(IdentifierContainer icontainer){
		if(graph == null){
			graph = new Graph(false);
			graph.getNodeTable().addColumn("name", String.class);
			graph.getNodeTable().addColumn("type", String.class);
			graph.getNodeTable().addColumn("desc", String.class);
			graph.getNodeTable().addColumn("identifier", Object.class);
		}
		
		if(icontainer == null)
			return graph;
				
	    for (Identifier ident : icontainer){
	    	String key = ident.getName()+"\r\n"+ident.getAttributes().toString();
	    	Identifier i = ident2identifier.get(key);
	    	if(i != null){	    			    		
	    		i.addLinks(ident.getLinks());
	    		Node n = ident2node.get(key);
	    		n.setString("desc", "<strong>Attributes</strong><br>"+i.getAttributes().toString());	    		
	    	}
	    	else {
	    		Node node = graph.addNode();	        	    	
	    		node.setString("name", key);
	    		node.setString("desc", "<strong>Attributes</strong><br>"+ident.getAttributes().toString());
	    		node.setString("type", "1");
	    		node.set("identifier", ident);
	    		ident2node.put(key, node);
    			ident2identifier.put(key, ident);
	    	}
	    }
	    
	    Hashtable<String, Link> hash = new Hashtable<String, Link>();
	    for (Identifier ident : icontainer){
	    	for(LinkContainer linkc : ident.getLinks()){
	    		for(Link link : linkc.getLinks()){
	    			hash.put(link.getHash(), link);	    			
	    		}
	      }
	    }
	    
	    Enumeration<String> enu = hash.keys();
	    
	    while(enu.hasMoreElements()){
	    	String key = enu.nextElement();
	    	Link link = hash.get(key);
	    	if(link2node.containsKey(link.getUniqueLinkName())){	    		
	    		if(link.isSingleValue()){		    			
	    			Node old = link2node.get(link.getUniqueLinkName());
					if(old != null){						
						old.setString("desc", link.toString());
						old.set("identifier", link);
						vis.run("repaint");
				        vis.run("color");
					}	    		
	    		}
	    		else{
	    			Node n = graph.addNode();
					n.setString("name", link.getName());
					n.setString("type", "0");
					n.setString("desc", link.toString());
					n.set("identifier", link);
					link2node.put(link.getUniqueLinkName(), n);
					link2link.put(link.getHash(), link);	
					String k1 = link.getIdentifier1().getName()+"\r\n"+link.getIdentifier1().getAttributes().toString();
					String k2 = link.getIdentifier2().getName()+"\r\n"+link.getIdentifier2().getAttributes().toString();
					graph.addEdge(ident2node.get(k1), n);
					graph.addEdge(ident2node.get(k2), n);	    		
	    		}
	    	}
	    	else{
	    		Node n = graph.addNode();
				n.setString("name", link.getName());
				n.setString("type", "0");
				n.setString("desc", link.toString());
				n.set("identifier", link);
			//n.set(VisualItem.SHAPE, Constants.SHAPE_DIAMOND);
				link2node.put(link.getUniqueLinkName(), n);
				link2link.put(link.getHash(), link);			
				String k1 = link.getIdentifier1().getName()+"\r\n"+link.getIdentifier1().getAttributes().toString();
				String k2 = link.getIdentifier2().getName()+"\r\n"+link.getIdentifier2().getAttributes().toString();
				graph.addEdge(ident2node.get(k1), n);
				graph.addEdge(ident2node.get(k2), n);
	    	}
	    }				
	    return graph;
	}
	
	private void showMetaInTable(VisualItem item){
		if(table != null){
			DefaultTableModel model = new DefaultTableModel();
			if(item.get("identifier") instanceof Identifier){
				Identifier ident = (Identifier)item.get("identifier");			
				String[] columns = new String[] { "Link", "Identifier 2", "Publisher-ID", "Timestamp", "Cardinality" };
				for(String s : columns){
					model.addColumn(s);
				}
				for(LinkContainer lc : ident.getLinks()){
					for(Link l : lc.getLinks()){
						String ident_name;
						if(l.getIdentifier1().equals(ident)){
							ident_name = l.getIdentifier2().getName();
						}
						else ident_name = l.getIdentifier1().getName();
						
						String[] row = {l.getName(), ident_name,
								l.getAttribute("ifmap-publisher-id"),
								l.getAttribute("ifmap-timestamp"),
								l.getAttribute("ifmap-cardinality")};
						model.addRow(row);
					}
				}
			}				
			else if(item.get("identifier") instanceof Link){
				Link link = (Link)item.get("identifier");
				String[] columns = new String[] { "Attributes", "Value" };
				for(String s : columns){
					model.addColumn(s);
				}
				
				Hashtable<String, Hashtable<String, String>> meta = link.getMetadata();
				Enumeration<String> keys = meta.keys();
				while(keys.hasMoreElements()){
					String key = keys.nextElement();					
					String[] row = {"<html><strong>"+key+"</strong></html>"};
					model.addRow(row);
					Hashtable<String, String> values = meta.get(key);
					Enumeration<String> value_keys = values.keys();
					while(value_keys.hasMoreElements()){
						String vkey = value_keys.nextElement();
						String[] vrow = {vkey, values.get(vkey)};
						model.addRow(vrow);
					}
				}
			
			}	
			
			table.setModel(model);	
		}
	}
	
	private void init(IdentifierContainer icontainer){
		Graph graph = fillGraph(icontainer);
	    // Visualisation
	    vis = new Visualization();
	    vis.add("graph", graph);
	   // vis.setInteractive("graph.edges", null, false);
	   
	    LabelRenderer labelRend = new LabelRenderer("name");
	    labelRend.setRoundedCorner(4, 4);
	    labelRend.setHorizontalPadding(4);
	    labelRend.setVerticalPadding(4);
	   // ShapeRenderer shape_renderer = new ShapeRenderer(20);
	  //  Shape shape = shape_renderer.ellipse(getX(), getY(), 6, 4);
	    
	   
	    EdgeRenderer edgeRend = new EdgeRenderer(Constants.EDGE_TYPE_LINE, Constants.EDGE_ARROW_NONE);	  
	    
	    vis.setRendererFactory(new DefaultRendererFactory(labelRend, edgeRend));
	    
	    TupleSet focusGroup = vis.getGroup(Visualization.FOCUS_ITEMS); 
        focusGroup.addTupleSetListener(new TupleSetListener() {
            public void tupleSetChanged(TupleSet ts, Tuple[] add, Tuple[] rem)
            {
                for ( int i=0; i<rem.length; ++i ){
                	VisualItem item = (VisualItem)rem[i];
                    item.setFixed(false);    
                    item.setStrokeColor(ColorLib.rgb(0, 0, 0));
                   // item.setStroke(new BasicStroke(1));
                }
              
                for ( int i=0; i<add.length; ++i ) {
                	VisualItem item = (VisualItem)add[i];
                    item.setFixed(true);
                    item.setStrokeColor(ColorLib.rgb(0, 255, 0));
                 //   item.setStroke(new BasicStroke( 1,
                  	//	  BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
                  		//  1, new float[]{ 1 }, 0 ));
                }
                if ( ts.getTupleCount() == 0 ) {
                	//System.err.println("SET fixed = false");
                    ts.addTuple(rem[0]);
                    ((VisualItem)rem[0]).setFixed(false);
                }   
               // System.err.println("tuple");
                //vis.run("color");
                if(animation)
                	vis.run("layout");
            }
        });
	    
	  //  ColorAction fillColor = new ColorAction("graph.nodes",
	    //        VisualItem.FILLCOLOR, ColorLib.rgb(255, 255, 128));
	    ColorAction borderColor = new ColorAction("graph.nodes",
	            VisualItem.STROKECOLOR, ColorLib.rgb(0, 0, 0));
	    ColorAction textColor = new ColorAction("graph.nodes",
	        VisualItem.TEXTCOLOR, ColorLib.rgb(0, 0, 0));	  
	    ColorAction edgeColor = new ColorAction("graph.edges",
	            VisualItem.STROKECOLOR, ColorLib.rgb(0, 0, 0));
	    ColorAction arrowColor = new ColorAction("graph.edges",
	        VisualItem.FILLCOLOR, ColorLib.rgb(0, 0, 0));
	    
	    int[] palette = new int[] {
	    	    ColorLib.rgb(252, 201, 8), ColorLib.rgb(134, 197, 241)
    	};
	    	// map nominal data values to colors using our provided palette
	    DataColorAction fill = new DataColorAction("graph.nodes", "type",
	    	    Constants.ORDINAL, VisualItem.FILLCOLOR, palette);

	    
	    ActionList color = new ActionList();
	    //color.add(fillColor);	 
	    color.add(fill);
	    color.add(borderColor);
	    color.add(textColor);
	    color.add(edgeColor);
	    color.add(arrowColor);

	    ActionList stroke = new ActionList();    
	    stroke.add(new StrokeAction("graph.edges", new BasicStroke(2)));
	    
	    ActionList layout = new ActionList(Activity.INFINITY);
	    //Activity.INFINITY
	  /*  ForceDirectedLayout force = new ForceDirectedLayout("graph", true){
	    	protected float getSpringLength(EdgeItem e) {
	            return 100;
	        }
	    };*/
	    ForceDirectedLayout forceLayout = new ForceDirectedLayout("graph", false);
	    //forceLayout.setIterations(3);
	   // forceLayout.setStepTime(30);
	      //forceLayout.getForceSimulator().getForces()[0].setParameter(NBodyForce.GRAVITATIONAL_CONST, -8);
	    //  forceLayout.getForceSimulator().getForces()[0].setParameter(NBodyForce.MIN_DISTANCE, 100f);
	      //forceLayout.getForceSimulator().getForces()[0].setParameter(NBodyForce.BARNES_HUT_THETA, 100);
	     //forceLayout.getForceSimulator().getForces()[1].setParameter(DragForce.DRAG_COEFF, 10);
	    //  System.err.println("MIN_DISTANCE:"+NBodyForce.);
	     // forceLayout.getForceSimulator().getForces()[2].setParameter(SpringForce.SPRING_COEFF, 0.2f);
	     // forceLayout.getForceSimulator().getForces()[2].setParameter(SpringForce.SPRING_LENGTH, 200f);
	    ForceSimulator fsim = new ForceSimulator(new RungeKuttaIntegrator());
	    float gravConstant = -40f; // the more negative, the more repelling
	    float minDistance = 300f; // -1 for always on, the more positive, the more space between nodes
	    float theta = 0.3f; // the lower, the more single-node repell calculation
	    float drag = 0.01f;
	    float springCoeff = 1E-4f; //1E-4
	    fsim.addForce(new NBodyForce(gravConstant, minDistance, theta));
	    fsim.addForce(new DragForce(drag));
	    fsim.addForce(new SpringForce(springCoeff, 5));
	    fsim.getForces()[2].setParameter(SpringForce.SPRING_LENGTH, 150f);
	    forceLayout.setForceSimulator(fsim);
	    layout.add(forceLayout);
	      
	   //   layout.setDuration(5000);
	   // FruchtermanReingoldLayout fr = new FruchtermanReingoldLayout("graph");
	 //   layout.add(fr);
	    
	      ActionList repaint = new ActionList();
	      repaint.add(new RepaintAction());
	      vis.putAction("repaint", repaint);

	    
	    //new BalloonTreeLayout("graph", 30)
	    layout.add(new RepaintAction());
	    
	   // DataShapeAction  data = new DataShapeAction("name");
	   // vis.putAction("data", data);

	   
	    vis.putAction("color", color);
	    vis.putAction("stroke", stroke);
	    vis.putAction("layout", layout);
	  //  vis.putAction("shapeaction", shapeaction);
	  
	    // create display and controls
	    
	    this.zoom(getLocation(), 0.9);
	    
	    setVisualization(vis);
	    //setSize(1024, 768);
	   	    
	    addControlListener(new SubtreeDragControl());
	    addControlListener(new PanControl()); 
	    addControlListener(new ZoomControl());    
	    addControlListener(new ZoomToFitControl());
	    addControlListener(new FocusControl(2, "layout"));	
	    addControlListener(new WheelZoomControl());
	    addControlListener(new NeighborHighlightControl());
	    
	    this.addControlListener(new ControlAdapter(){	    	
	    	public void itemPressed(VisualItem item, MouseEvent e) {
	    		synchronized(this){	    
	    		if(selectedNode != null){	    			
	    			if(selectedNode.get("identifier") instanceof Identifier){
	    				selectedNode.setFillColor(ColorLib.rgb(134, 197, 241));
	    			}
	    			else selectedNode.setFillColor(ColorLib.rgb(252, 201, 8));
	    		}
        		item.setFillColor(ColorLib.rgb(255, 0, 0));        		
        		selectedNode = item;
        		showMetaInTable(item);
        		notifyNodeSelectedReceiver(item);
        		//vis.run("color");
        		//vis.run("stroke");
        		//vis.run("repaint");
	    		}
	    	}
	    });
	    
	   // display.addControlListener(new AggregateDragControl());
        setHighQuality(true);
        setBackground(Color.WHITE);
        ToolTipControl ttc = new ToolTipControl("name"){
        	public void itemEntered(VisualItem item, java.awt.event.MouseEvent e){
        		Display d = (Display)e.getComponent();
        		//JCustomTooltip tooltip = new JCustomTooltip(d, 
        			//	new JLabel("<html><p>BLA!</p></html>"));
        		//d.setToolTipText((String)item.get("desc"));
        		//d.setCustomToolTip(tooltip);
        		String desc = (String)item.get("desc");
        		desc = desc.replace("\r\n", "<BR>");
        		d.setToolTipText("<html>"+desc+"</html>");
        	}
        };
        addControlListener(ttc);

        
	    vis.run("color");
	    vis.run("stroke");
	    if(animation)
	    	vis.run("layout");
	    
 }
	
/*	public void addNode(Identifier ident){
		//Node newNode = graph.addNode();
		 Node node = graph.addNode();	        
	     node.setString("name", ident.getName()+"\r\n"+ident.getAttributes().toString());
	        node.setString("type", "1");
	        ident2node.put(ident, node);
	        System.err.println("JPrefuse.getJComponent: Füge Identifier ein");
	        vis.run("repaint");
	        vis.run("color");
	        vis.run("layout");

		//graph.addEdge(oldNode, newNode);
	}*/
	
	public void toggleAnimation(){
		if(animation) {
			enableAnimation(false);
			this.animation = false;
			} 
		else {
			enableAnimation(true);
			this.animation = true;
		}
	}
	
	private void enableAnimation(boolean b){
		if(b){
			if(!vis.getAction("layout").isRunning()){
				vis.run("layout");
			}
		}
		else {
			vis.cancel("layout");
		}
	}
	
	public synchronized void addIdentifier(IdentifierContainer icontainer){
		this.fillGraph(icontainer);
		vis.run("repaint");
        vis.run("color");
        //System.err.println("add identifier:"+icontainer.toString());
       // vis.run("layout");
	}
	
	public int countIdentifier(){
		return ident2node.size();
	}
	
	public int countLinks(){
		return link2node.size();
	}
	
	public synchronized void deleteIdentifier(IdentifierContainer icontainer){
		for (Identifier ident : icontainer){
			String key = ident.getName()+"\r\n"+ident.getAttributes().toString();
	    	Identifier i = ident2identifier.get(key);
	    	if(i != null){	    		    		
	    	    	for(LinkContainer linkc : i.getLinks()){
	    	    		for(Link link : linkc.getLinks()){
	    	    			Node node = link2node.get(link.getUniqueLinkName());
	    	    			if(selectedNode != null && selectedNode.get("identifier") != null){	    			
    	    					if(selectedNode.get("identifier") instanceof Link){
    	    						Link sel = (Link)selectedNode.get("identifier");
    	    						if(link.equals(sel)){
    	    							this.table.setModel(new DefaultTableModel());
    	    							selectedNode = null;
    	    							if(label != null){
    	    								label.setText("");
    	    							}
    	    					 }	    	    						
    	    				   }	    	    					
    	    				}
	    	    			if(node != null){
	    	    				graph.removeNode(node);
	    	    				link2node.remove(link.getUniqueLinkName());
	    	    				link2link.remove(link.getHash());	    	    					    	    					    	    				
	    	    			}
	    	    		}
	    	    	}	    	    	    	    	    	   
	    		Node n = ident2node.get(key);
	    		if(n != null){
	    			if(selectedNode != null && selectedNode.get("identifier") != null){	    			
    					if(selectedNode.get("identifier") instanceof Identifier){
    						Identifier sel = (Identifier)selectedNode.get("identifier");
    						if(i.isTheSameLike(sel)){
    							this.table.setModel(new DefaultTableModel());
    							selectedNode = null;
    							if(label != null){
    								label.setText("");
    							}
    					 }	    	    						
    				   }	    	    					
    				}	
	    			graph.removeNode(n);
	    			ident2identifier.remove(key);
	    			ident2node.remove(key);	    			
	    		}
	    	}
	    	
		}
		vis.run("repaint");
        vis.run("color");
	}

	@Override
	public void componentHidden(ComponentEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentMoved(ComponentEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentResized(ComponentEvent arg0) {
		revalidate();
        pan(getParent().getWidth()/2, getParent().getHeight()/2);
	}

	@Override
	public void componentShown(ComponentEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void processDelete(IdentifierContainer conti) {
		this.deleteIdentifier(conti);
	}

	@Override
	public void processNew(IdentifierContainer conti) {
		this.addIdentifier(conti);
		//System.err.println("JPrefuse: processNew!");
		
	}

	@Override
	public void processUpdate(IdentifierContainer conti) {
		this.addIdentifier(conti);
	}
}