/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.visual
 * File:    MyFrame.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.visual;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Observable;
import java.util.Observer;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import prefuse.visual.VisualItem;
import de.fhhannover.inform.ifmap.control.IfmapFacade;
import de.fhhannover.inform.ifmap.datastructure.Identifier;
import de.fhhannover.inform.ifmap.datastructure.IdentifierContainer;
import de.fhhannover.inform.ifmap.datastructure.Link;
import de.fhhannover.inform.ifmap.event.IdentifierChangedReceiver;
import de.fhhannover.inform.ifmap.event.NodeSelectedReceiver;
import de.fhhannover.inform.ifmap.event.StatusChangedReceiver;
import de.fhhannover.inform.ifmap.exception.PropertiesNotFoundException;
import de.fhhannover.inform.ifmap.output.JPrefuse;

/**
 * This code was edited or generated using CloudGarden's Jigloo SWT/Swing GUI
 * Builder, which is free for non-commercial use. If Jigloo is being used
 * commercially (ie, by a corporation, company or business for any purpose
 * whatever) then you should purchase a license for each developer using Jigloo.
 * Please visit www.cloudgarden.com for details. Use of Jigloo implies
 * acceptance of these licensing terms. A COMMERCIAL LICENSE HAS NOT BEEN
 * PURCHASED FOR THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED LEGALLY FOR
 * ANY CORPORATE OR COMMERCIAL PURPOSE.
 */
@SuppressWarnings("serial")
public class MyFrame extends javax.swing.JFrame implements Observer{

	int windowWidth = 1024;
	int windowHeight = 768;
	int btnWidth = 146;
	int btnHeight = 38;
	int sepWidth = 32;
	int sepHeight = 42;

	private IfmapFacade dump;
	
	{
		// Set Look & Feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager
					.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private class DumpHelper extends Thread implements StatusChangedReceiver,
			NodeSelectedReceiver {
		private boolean pause = false;
		boolean active = true;
		private long lastUpdate = 0;		
		

		public void pause(boolean b) {
			pause = b;
		}

		public boolean isPaused() {
			return pause;
		}

		public void processStatusEvent(String string) {
			statusMessageLabel.setText(string);
		}

		@Override
		public void run() {
			JPrefuse prefuse = null;
			dump = IfmapFacade.getInstance("ifmap.properties");
			dump.addStatusListener(this);

			if (dump != null) {				
				if (graphPanel.getComponentCount() == 0) {
					prefuse = (JPrefuse) dump.getPrefuseGraph();
					prefuse.setTable(jTable1, jLabel1);
					prefuse.addNodeSelectedListener(this);
					dump.addUpdatesListener(prefuse);
					graphPanel.add(prefuse, BorderLayout.CENTER);
					graphPanel.revalidate();
					prefuse.revalidate();
					prefuse.pan(prefuse.getParent().getWidth() / 2, prefuse
							.getParent().getHeight() / 2);
				}
			}	

			while (active) {				
				if (!pause) {
					long time = 0;
					try {
						time = dump.getLastUpdateTime();						
						if (dump.getPublisherID() != null
								&& dump.getSessionID() != null) {
							setTitle("Publisher-ID: " + dump.getPublisherID()
									+ " / Session-ID: " + dump.getSessionID());
						}
						if (time > lastUpdate) {
							lastUpdate = time;
							IdentifierContainer conti = null;
							conti = dump.getSearchIdentifier();
							if (conti != null && conti.size() > 0) {
								dump.subscribe(conti);
								dump.enablePolling(true);
							}
						}												

					} catch (PropertiesNotFoundException ex) {
						System.err.println(ex.getMessage());
						System.exit(ERROR);
					} catch (RemoteException ex) {
						System.err.println(ex.getMessage());
						//System.exit(ERROR);
					}
				}								
				
				try {
					//timestampLabel.setText("Identifier: "+prefuse.countIdentifier()+"  Links: "+prefuse.countLinks());
					Thread.sleep(5000);
				} catch (Exception err) {
					err.printStackTrace();
				}
			}

		}

		@Override
		public void nodeSelected(VisualItem item) {
			if (item.get("identifier") instanceof Identifier) {
				Identifier ident = (Identifier) item.get("identifier");
				jLabel1.setText("<html><strong><font face='verdana' size='4'>"
						+ ident.getName() + "</font></strong>  "
						+ ident.getAttributes() + "</html>");
			} else if (item.get("identifier") instanceof Link) {
				Link link = (Link) item.get("identifier");
				jLabel1.setText("<html><strong><font face='verdana' size='4'>"
						+ link.getName() + "</font></strong>  "
						+ link.getAttributes().toString() + "</html>");
			}

		}
	};

	private JToolBar jToolBar1;
	private JToggleButton jButton1;
	private JSplitPane jSplitPane1;
	private JButton saveButton;
	private JLabel timestampLabel;
	private JLabel statusMessageLabel;
	private JPanel statusPanel;
	private JScrollPane jScrollPane1;
	private JLabel logo_tnc;
	private JPanel jPanel1;
	private JLabel logo;
	private JButton jButton2;
	private JToggleButton jToggleButton1;
	private JLabel jLabel1;
	private JPanel jPanel3;
	private JPanel jPanel2;
	private JTable jTable1;
	private JPanel graphPanel;
	private DumpHelper dh;
	private SubscribeObservable subscription_gui = new SubscribeObservable();

	public void startIfmap() {
		dh.pause(true);
		dh.start();
	}

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MyFrame inst = new MyFrame();
				inst.setLocationRelativeTo(null);
				inst.setVisible(true);
				inst.startIfmap();
			}
		});
	}

	public MyFrame() {
		super();
		initGUI();
		this.subscription_gui.addObserver(this);
		dh = new DumpHelper();
		this.jScrollPane1.getViewport().setBackground(Color.WHITE);
	}

	private void initGUI() {
		try {
			this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			getContentPane().setBackground(new java.awt.Color(255, 255, 255));
			this.setFocusable(false);
			{
				jToolBar1 = new JToolBar();
				getContentPane().add(jToolBar1, BorderLayout.NORTH);
				jToolBar1.setBackground(new java.awt.Color(240, 240, 240));
				jToolBar1.setFloatable(false);
				jToolBar1.setMargin(new java.awt.Insets(3, 3, 3, 3));				
				{
					jButton1 = new JToggleButton();
					jToolBar1.add(jButton1);
					jToolBar1.addSeparator(new Dimension(sepWidth, sepHeight));
					jButton1.setText("Start/Stop");
					jButton1.setFocusPainted(false);
					jButton1.setFocusable(false);
					jButton1.setIcon(new ImageIcon(getClass().getClassLoader().getResource("connect.png")));
					jButton1.setPreferredSize(new java.awt.Dimension(btnWidth, btnHeight));
					jButton1.setSize(btnWidth, btnHeight);
					jButton1.setToolTipText("Start / Stop the periodically sending of dump-messages");
					jButton1.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent evt) {
							if(!dh.isAlive()){
								dh.start();
							}
							if (dh.isPaused())
								dh.pause(false);
							else{
								dh.pause(true);								
							}
						}
					});
				}
				{
					jToggleButton1 = new JToggleButton();
					jToolBar1.add(jToggleButton1);
					jToolBar1.addSeparator(new Dimension(sepWidth, sepHeight));
					jToggleButton1.setText("Animation On/Off");
					jToggleButton1.setSelected(true);
					jToggleButton1.setFocusable(false);
					jToggleButton1.setIcon(new ImageIcon(getClass().getClassLoader().getResource("animation.png")));
					jToggleButton1.setPreferredSize(new java.awt.Dimension(btnWidth, btnHeight));
					jToggleButton1.setSize(btnWidth, btnHeight);
					jToggleButton1.setToolTipText("Turns the animation On/Off");
					jToggleButton1.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent evt) {
							animationButtonActionPerformed(evt);
						}
					});
				}
				{
					saveButton = new JButton();
					jToolBar1.add(saveButton);
					jToolBar1.addSeparator();
					saveButton.setText("Snapshot");
					saveButton.setPreferredSize(new java.awt.Dimension(btnWidth, btnHeight));
					saveButton.setFocusable(false);
					saveButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("snapshot.png")));
					saveButton.setSize(btnWidth, btnHeight);
					saveButton.setToolTipText("Save the actual view as PNG");
					saveButton.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent evt) {
							saveButtonActionPerformed(evt);
						}
					});
				}
				{
					jButton2 = new JButton();
					jToolBar1.add(jButton2);
					jButton2.setText("Subscribe...");
					jButton2.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent evt) {
							if(subscription_gui == null)
								subscription_gui = new SubscribeObservable();
							subscription_gui.showGUI();
						}
					});
				}
				{
					jPanel1 = new JPanel();
					FlowLayout jPanel1Layout = new FlowLayout();
					jPanel1Layout.setAlignment(FlowLayout.RIGHT);
					jPanel1.setLayout(jPanel1Layout);
					jToolBar1.add(jPanel1);
					int tmpWidth = windowWidth - 3 * btnWidth - 3 * sepWidth;
					jPanel1.setPreferredSize(new java.awt.Dimension(tmpWidth, 46));
					jPanel1.setSize(tmpWidth, 46);
					{
						logo_tnc = new JLabel();
						jPanel1.add(logo_tnc);
						logo_tnc.setIcon(new ImageIcon(getClass().getClassLoader().getResource("logo_trustfhh.png")));
						logo_tnc.setToolTipText("Visit the Trust@FHH Website!");
						logo_tnc.addMouseListener(new MouseAdapter() {
							public void mouseExited(MouseEvent evt) {
								logo_tnc.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							}
							public void mouseEntered(MouseEvent evt) {
								logo_tnc.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
							}
							public void mouseClicked(MouseEvent evt) {
								de.fhhannover.inform.ifmap.util.AppLauncher.openWebpageInBrowser("http://trust.inform.fh-hannover.de");
							}
						});
					}
					{
						logo = new JLabel();
						jPanel1.add(logo);
						logo.setIcon(new ImageIcon(getClass().getClassLoader().getResource("logo_iron.png")));
						logo.setToolTipText("More interesting IFMAP-stuff");
						logo.addMouseListener(new MouseAdapter() {
							public void mouseExited(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							}
							public void mouseEntered(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
							}
							public void mouseClicked(MouseEvent evt) {
								de.fhhannover.inform.ifmap.util.AppLauncher.openWebpageInBrowser("http://trust.inform.fh-hannover.de/joomla/index.php/projects/iron");
							}
						});
					}
				}
			}
			{
				jSplitPane1 = new JSplitPane();
				GridLayout jSplitPane1Layout = new GridLayout(1, 2);
				jSplitPane1Layout.setColumns(2);
				jSplitPane1.setLayout(jSplitPane1Layout);
				getContentPane().add(jSplitPane1, BorderLayout.CENTER);
				jSplitPane1.setDividerLocation(375);
				jSplitPane1.setBackground(new java.awt.Color(255, 255, 255));
				jSplitPane1.setOrientation(JSplitPane.VERTICAL_SPLIT);
				{
					graphPanel = new JPanel();
					jSplitPane1.add(graphPanel, JSplitPane.TOP);
					BorderLayout graphPanelLayout = new BorderLayout();
					graphPanel.setLayout(graphPanelLayout);
					graphPanel.setBackground(new java.awt.Color(255, 255, 255));
					graphPanel
							.setPreferredSize(new java.awt.Dimension(775, 259));
				}
				{
					jPanel2 = new JPanel();
					BorderLayout jPanel2Layout = new BorderLayout();
					jPanel2.setLayout(jPanel2Layout);
					jSplitPane1.add(jPanel2, JSplitPane.BOTTOM);
					{
						jPanel3 = new JPanel();
						FlowLayout jPanel3Layout = new FlowLayout();
						jPanel2.add(jPanel3, BorderLayout.NORTH);
						jPanel3
								.setPreferredSize(new java.awt.Dimension(390,
										22));
						jPanel3.setLayout(jPanel3Layout);
						jPanel3Layout.setAlignment(FlowLayout.LEFT);
						jPanel3Layout.setAlignment(FlowLayout.LEFT);
						{
							jLabel1 = new JLabel();
							jPanel3.add(jLabel1);
							jLabel1.setHorizontalAlignment(SwingConstants.LEFT);
							jLabel1
									.setHorizontalTextPosition(SwingConstants.LEFT);
						}
					}
					{
						jScrollPane1 = new JScrollPane();
						jPanel2.add(jScrollPane1, BorderLayout.CENTER);
						jScrollPane1.setPreferredSize(new java.awt.Dimension(
								780, 108));
						jScrollPane1.setBackground(new java.awt.Color(255, 255,
								255));
						{
							jTable1 = new JTable();
							jScrollPane1.setViewportView(jTable1);
							jTable1.setCellSelectionEnabled(true);
							jTable1.setBackground(new java.awt.Color(255, 255,
									255));
							jTable1
									.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
							jTable1.setAutoCreateRowSorter(true);
						}
					}
				}
			}
			{
				statusPanel = new JPanel();
				BorderLayout statusPanelLayout = new BorderLayout();
				getContentPane().add(statusPanel, BorderLayout.SOUTH);
				statusPanel.setPreferredSize(new java.awt.Dimension(777, 18));
				statusPanel.setLayout(statusPanelLayout);
				{
					statusMessageLabel = new JLabel();
					statusPanel.add(statusMessageLabel, BorderLayout.CENTER);
					statusMessageLabel.setPreferredSize(new java.awt.Dimension(
							645, 18));
					statusMessageLabel
							.setHorizontalAlignment(SwingConstants.RIGHT);
				}
				{
					timestampLabel = new JLabel();
					statusPanel.add(timestampLabel, BorderLayout.WEST);
					timestampLabel.setPreferredSize(new java.awt.Dimension(249,
							18));
				}
			}
			pack();
			this.setSize(1024, 768);
		} catch (Exception e) {
			// add your error handling code here
			e.printStackTrace();
		}
	}

	private void saveButtonActionPerformed(ActionEvent evt) {
		final JFileChooser fc = new JFileChooser();
		int width = this.graphPanel.getWidth();
		int height = this.graphPanel.getHeight();
		BufferedImage image = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g = image.createGraphics();
		this.graphPanel.paint(g);
		g.dispose();
		try {
			int returnVal = fc.showSaveDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				ImageIO.write(image, "png", fc.getSelectedFile());
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void animationButtonActionPerformed(ActionEvent evt) {
		if (graphPanel.getComponentCount() > 0) {
			if (graphPanel.getComponent(0) instanceof JPrefuse) {
				JPrefuse prefuse = (JPrefuse) graphPanel.getComponent(0);
				prefuse.toggleAnimation();
			}
		}
	}

	@Override
	public void update(Observable arg0, Object arg1) {		
		SubscribeObservable so = (SubscribeObservable) arg0;
		try {
			if(dump.connect()){
				boolean b = dump.subscribe(so.getIdentifier());	
			if(b)
				{
					statusMessageLabel.setText("Subscribe erfolgreich!");					
					dump.enablePolling(true);
				}						
			else dh.processStatusEvent("Subscribe erfolglos!");
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PropertiesNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

}
